# Quickstart

Prism is an artificial exchange in which participants (agents) can submit trade orders via an API. This guide
will help you setup your Python environment and make your first few calls to the Prism server.

## Setup

### 1. Install Poetry
Prism uses [Poetry](https://python-poetry.org) to manage Python dependencies. The first step is 
to install Poetry on your machine using the instructions [here](https://python-poetry.org/docs/).

### 2. Create a Python Virtual Environment
You can use your favorite Python Virtual Environment manager. Here we provide an example with [conda](anaconda.org).
The Python Version in Prism is `3.9`. You can also verify the version used through `pyproject.toml` included in your starter kit.

```
$ conda create -n prism python==3.9
$ source activate prism
```

### 3. Install Python Dependencies
Now Run:
```
### Install dependencies from the poetry.lock
$ poetry install
```

If you wish to run the included Jupyter notebooks for tutorials, additionally run the following.
```
python -m ipykernel install --user --name prism
```

## Connect to The Prism API
In order to contact the Prism API, you will need:
1. The Prism Server URL:
    - You can get this by signing into [Prism](https://prism.borealisai.com)
    - Under `Choose Your Instance`, select `Test Instance`. Note: When you're participating in a live competition, you can use the competition instance.
    - Click on `API Link` and the link is copied.
2. Your Access Token:
    - Next to the `API Link` from the previous step is another field `Access Token`.
    - Click on it, and your token is copied.
3. Your Alias:
    - Prism uses your alias to represent you on the Prism Leaderboard.
    - Select an alias of your choosing under 32 characters. Emojis are permitted within the alias.

Now we're ready to make our first call to Prism. Register yourself by issuing the folling Curl request:
```
$ curl -X POST <test-server-url>/api/v1/accounts -d "{\"alias\": \"<your-alias>\"}" --header "Content-Type: application/json" --header "Authorization: Bearer <your-access-token>"
> "5d11fc1f-aa8a-4b83-945d-802e1790b9ce"  # <-- Your Client ID!
```

The call returns a unique client id to be used in future requests to Prism.

Congratulations! You just made your first API call to Prism and are ready for next steps. 

## Next Steps
You're ready to create your own trading agent! 

You can choose to write your own scripts using:
- the Prism API schema (click "Swagger UI" on the Prism Dashboard), or
- you can use our [starter kit](./starter_kit.md) along with [tutorials](./tutorials/) to help you develop your own agent.
