from typing import Union

import attr
from attr import frozen


@attr.s(auto_attribs=True, frozen=True)
class Order:
    volume: int = attr.ib()

    @volume.validator
    def check(self, attribute, value):
        assert self.volume > 0


@attr.s(auto_attribs=True, frozen=True)
class BuyMarketOrder(Order):
    pass


@attr.s(auto_attribs=True, frozen=True)
class SellMarketOrder(Order):
    pass


@attr.s(auto_attribs=True, frozen=True)
class PlaceBidLimitOrder(Order):
    index: int


@attr.s(auto_attribs=True, frozen=True)
class PlaceAskLimitOrder(Order):
    index: int


@frozen
class CancelBidLimitOrder:
    index: int


@frozen
class CancelAskLimitOrder:
    index: int


CSTEvent = Union[
    BuyMarketOrder,
    SellMarketOrder,
    PlaceBidLimitOrder,
    PlaceAskLimitOrder,
    CancelBidLimitOrder,
    CancelAskLimitOrder,
]
