# Fundamentalist Agents
Fundamentalist agents value stocks not by using market prices, by reference to an external fundamental value process.  In many models, fundamentalist agents stabilize market prices, while chartist agents (which forecast based on market price history) destabilize market prices.  See https://arxiv.org/pdf/0808.3562.pdf.

To implement a Fundamentalist agent, subclass FundamentalState and override "advance" with functionality for advancing the process one step, and "conditional_expectation" with functionality for computing the conditional expectation of the process.  Then use this FundamentalState to create a Fundamentalist forecaster and combine it with a TradingRule to create a ForecastRunner agent.  This approach has two main advantages:

a) It separates the logic for the evolution of the process from the logic for submitting orders.

b) By forcing all processes to conform to the same interface, we can easily combine them into higher-level processes like regime-switching models.

Here is an example of using this approach to create an OU-fundamentalist agent:

```
  ou_forecast = Fundamentalist(
        start_state=OrnsteinUhlenbeckState.create_with_half_life(
            half_life=2, sigma=5, step=0, value=50, b=50
        ),
    )
    rule = SpreadTradingRule(
        volume=10,
    )
    ou_trader = ForecastRunner(
        forecaster=ou_forecast,
        trading_rule=rule,
        horizon=1,
        symbol="PRSM",
        alias="ou",
        period=1,
    )
```
Helper functions can be writen to reduce boilerplate.