from typing import Optional
from uuid import UUID

import numpy as np

from prism_agents.agents.background.annealing_schedule import AnnealingState, ConstantAnnealingState
from prism_agents.agents.background.poisson_fundamental_trader import create_poisson_fundamentalist
from prism_agents.agents.forecast.forecasters.fundamentalist.process.kim_omberg import KOState


def create_ko_poisson_agent(
    seed: int,
    sigma: float,
    start_value: float,
    drift_alpha: float,
    drift_b: float,
    drift_sigma: float,
    fundamental_period: float,
    limit_arrival_rate: int,
    cancel_factor: float = 0.5,
    alias="fundamental_trader",
    annealer: AnnealingState = ConstantAnnealingState(),
    client_id: Optional[UUID] = None,
    symbol: str = "PRSM",
):
    seed_1, seed_2 = np.random.SeedSequence(seed).spawn(2)
    state = KOState(
        value=start_value,
        current_drift=0.0,
        sigma=sigma,
        drift_alpha=drift_alpha,
        drift_sigma=drift_sigma,
        drift_b=drift_b,
        generator=np.random.default_rng(seed_1),
    )
    return create_poisson_fundamentalist(
        initial_state=state,
        limit_arrival_rate=limit_arrival_rate,
        fundamental_period=fundamental_period,
        alias=alias,
        seed=seed_2.generate_state(1)[0],
        cancel_factor=cancel_factor,
        annealer=annealer,
        cliend_id=client_id,
        symbol=symbol,
    )
