import logging

DEFAULT_LOG_FORMAT = "%(asctime)s: %(name)s - %(levelname)s - %(message)s"
DEFAULT_LOG_LEVEL = logging.INFO

"""
Copied and pasted from prism
"""


def prism_agents_configure_logging(
    logger: logging.Logger, format: str = DEFAULT_LOG_FORMAT, level: int = DEFAULT_LOG_LEVEL
) -> logging.Logger:
    """Configure the `logging` object for prism_agents.

    Args:
        logger (logging.Logger): The `logging` object to configure.
        format (str, optional): The format to apply to loggers. Defaults to LOG_FORMAT.
        level (int, optional): Logging verbosity. Defaults to DEFAULT_LOG_LEVEL.

    Returns:
        logging.Logger: The configured `logging` object.
    """
    logger.setLevel(level)
    formatter = logging.Formatter(format)
    handler: logging.Handler
    if not logger.handlers:
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    else:
        for handler in logger.handlers:
            handler.setFormatter(formatter)

    return logger
