from typing import Tuple

import attr
from torch.utils.data import Dataset


@attr.s(frozen=True, auto_attribs=True)
class TimestepInfo:
    timestep: int
    ask: float
    bid: float
    volume: int


class MarketDataset(Dataset):
    def get_next_market_state(self, time: int) -> Tuple[int, TimestepInfo]:
        """
        A function that returns a market state for a given dataset.
        There are no restriction on what this state is.

        :param time: int, used to keep track of current time
        :return: (next_time, market_dataset_information)
        """
        raise NotImplementedError

    def __len__(self):
        raise NotImplementedError
