from typing import Mapping, NamedTuple, Sequence

import attr

from prism_agents.api.agent_schemas import ClientDataTable, MarketData


@attr.s(frozen=True, auto_attribs=True)
class Observation:
    """
    A data point recorded during an experiment
    """

    market: Mapping[str, MarketData]
    client_data: ClientDataTable


class TimedObservation(NamedTuple):
    time: float
    observation: Observation


@attr.s(frozen=True, auto_attribs=True)
class ExperimentResult:
    """
    A Sequence of data points recorded during an experiment.
    Each data point is paired with a float, recording the relative time
    of the observation according to the experiment runner's internal clock.
    """

    observations: Sequence[TimedObservation]
