from typing import Sequence

from ray.util.multiprocessing import Pool
from typing_extensions import Literal

from prism_agents.api.server_interface import ServerInterface
from prism_agents.runner.experiment_result import ExperimentResult
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple
from prism_agents.runner.stepwise_agent_runner import StepwiseAgentRunner

run_mode = Literal["sequential", "parallel"]


def run_batch(
    spec: SimpleExperimentSpec,
    agent_sequences: Sequence[Sequence[StepwiseAgentRunner]],
    server_interface: ServerInterface,
    mode: run_mode = "sequential",
    parallel_workers: int = 4,
    chunksize: int = 1,
) -> Sequence[ExperimentResult]:
    """
    Run  experiments which differ by their agents.  Return a list of results.
    Note that it is advisable to  make sure the agents
     in each subsequence have different random seeds
    """
    if mode == "sequential":
        results = []
        for agents in agent_sequences:
            results.append(
                run_experiment_simple(
                    agents=agents, experiment_spec=spec, server_interface=server_interface
                )
            )
            server_interface.clear()
        return results

    def sub_run(
        run_agents: Sequence[StepwiseAgentRunner],
    ) -> ExperimentResult:
        return run_experiment_simple(
            experiment_spec=spec,
            agents=run_agents,
            server_interface=server_interface.spawn_episode(),
        )

    return Pool(processes=parallel_workers).map(sub_run, agent_sequences, chunksize=chunksize)
