import time
from decimal import Decimal
from pathlib import Path

import pytorch_lightning as pl
import wandb
from pytorch_lightning.loggers import WandbLogger

from prism_agents.agents.forecast.forecast_agent import ForecastRunner
from prism_agents.agents.forecast.forecasters.lstm.adaptive_lstm import AdaptiveLSTMForecaster
from prism_agents.agents.forecast.forecasters.lstm.nn import LightningLSTMForecastingModule
from prism_agents.agents.forecast.trading_rule.simple_trading_rule import SimpleTradingRule
from prism_agents.tracking.loading import load_from_wandb

if __name__ == "__main__":
    time_tag = str(time.time())[-3:]
    path = Path("sandbox/lstm_tests/lightning_example")

    run = wandb.init()
    torch_forecaster = load_from_wandb(
        cls=LightningLSTMForecastingModule,
        run=run,
        wandb_path="prism-agents/prism-agents/model-3n5xonut:v0",
        type="model",
    )

    # Define our adaptive update trainer
    chkpt = pl.callbacks.ModelCheckpoint()
    logger = WandbLogger(
        name="lightning_test",
        project="prism-agents",
        job_type="test",
        entity="prism-agents",
        log_model=True,
    )

    def get_trainer():
        trainer = pl.Trainer(
            max_epochs=3, callbacks=[chkpt], default_root_dir=str(path), logger=logger
        )
        return trainer

    forecaster = AdaptiveLSTMForecaster(forecaster=torch_forecaster, get_trainer=get_trainer)

    trading_rule = SimpleTradingRule(volume=10, omega=Decimal(3), strict=False)
    adaptive_forecast = ForecastRunner(
        forecaster=forecaster,
        trading_rule=trading_rule,
        horizon=1,
        alias=f"{time_tag}_AdaptiveLSTM",
        symbol="PRSM",
    )

    if True:
        from prism_agents.agents.zero_intelligence.market_replay import MarketReplay
        from prism_agents.api.remote_request_funcs import RemoteServerInterface
        from prism_agents.dataset.spy import SPYBidAskVolume
        from prism_agents.runner.experiment_runner import ExperimentSpec, run_experiment

        spy_dataset = SPYBidAskVolume(tiny=True)
        ra = MarketReplay(
            dataset=spy_dataset,
            order_request_delay=2,
            LOB_depth=5,
            time_init=0,
            client_alias="SPY_FT",
        )

        url = "http://francis-test-server-route-a5w0-photon.apps.borealis-rnd-scc.fg.rbc.com"
        server_interface = RemoteServerInterface(server_url=url)
        duration = 30
        run_experiment(
            agents=[ra, adaptive_forecast],
            experiment_spec=ExperimentSpec(
                duration=duration, time_warp_factor=1, num_observations=int(duration / 1)
            ),
            server_interface=server_interface,
        )
