import copy
import time
from pathlib import Path

import pytorch_lightning as pl
from attrs import asdict
from pytorch_lightning.callbacks.early_stopping import EarlyStopping
from pytorch_lightning.loggers import WandbLogger
from torch.utils.data import DataLoader

from prism_agents.agents.forecast.forecasters.lstm.nn import (
    LightningLSTMForecastingModule,
    LSTMForecastingModuleInputs,
)
from prism_agents.agents.utils.vector_simulation.processes import ou_process
from prism_agents.dataset.fundamental_process import FundamentalProcessDataset, InfoRange

if __name__ == "__main__":
    time_tag = str(time.time())[-3:]
    path = Path("sandbox/lstm_tests/lightning_example")

    input_range = {
        "step_size": InfoRange(mode="choice", options=[1]),
        "initial_values": InfoRange(mode="uniform", options=[30, 100]),
        "b": InfoRange(mode="uniform", options=[30, 100]),
        "sigma": InfoRange(mode="uniform", options=[1, 10]),
        "half_life": InfoRange(mode="uniform", options=[0.1, 10]),
    }

    """
    input_range = {
        "step_size": InfoRange(mode="choice", options=[1]),
        "initial_values": InfoRange(mode="uniform", options=[50, 50]),
        "b": InfoRange(mode="uniform", options=[50, 50]),
        "sigma": InfoRange(mode="uniform", options=[5, 5]),
        "half_life": InfoRange(mode="uniform", options=[2, 2]),
    }
    """

    observation_history = 20
    train_dataset = FundamentalProcessDataset(
        stochastic_process_generator=ou_process,
        input_range=input_range,
        batch_size=128,
        seq_length=observation_history,
        target_horizon=1,
    )
    valid_dataset = copy.deepcopy(train_dataset)
    valid_dataset.batches_per_epoch = 10

    train_dataloader = DataLoader(train_dataset, batch_size=None, sampler=None)
    valid_dataloader = DataLoader(valid_dataset, batch_size=None, sampler=None)

    seq_length = observation_history
    _inputs = LSTMForecastingModuleInputs(
        input_size=1,
        hidden_size=16,
        num_layers=3,
        seq_length=seq_length,
    )

    f = LightningLSTMForecastingModule(lr=1e-4, **asdict(_inputs))
    chkpt = pl.callbacks.ModelCheckpoint()
    early_stopping = EarlyStopping(monitor="val_loss", mode="min", patience=5)
    logger = WandbLogger(
        name="lightning_pretrained_lstm_OU",
        project="prism-agents",
        job_type="test",
        entity="prism-agents-examples",
        log_model=True,
    )
    trainer = pl.Trainer(
        max_epochs=30, callbacks=[chkpt, early_stopping], default_root_dir=str(path), logger=logger
    )
    trainer.fit(f, train_dataloaders=train_dataloader, val_dataloaders=valid_dataloader)
