import time
from pathlib import Path

import pytorch_lightning as pl
from attrs import asdict
from pytorch_lightning.loggers import WandbLogger

from prism_agents.agents.forecast.forecasters.lstm.dataset import SPYBidAskVolumeLSTMDataset
from prism_agents.agents.forecast.forecasters.lstm.nn import (
    LightningLSTMForecastingModule,
    LSTMForecastingModuleInputs,
)

if __name__ == "__main__":
    time_tag = str(time.time())[-3:]
    path = Path("sandbox/lstm_tests/lightning_example")

    dataset = SPYBidAskVolumeLSTMDataset(tiny=True, batch_size=200, target_shift=5)
    dataloader = dataset.make_batch_loader()

    seq_length = dataset[0]["input"].shape[0]
    _inputs = LSTMForecastingModuleInputs(
        input_size=2, hidden_size=5, num_layers=2, seq_length=seq_length
    )
    f = LightningLSTMForecastingModule(**asdict(_inputs))

    chkpt = pl.callbacks.ModelCheckpoint()
    logger = WandbLogger(
        name="lightning_test_adaptive_lstm",
        project="prism-agents",
        job_type="test",
        entity="prism-agents",
        log_model=True,
    )
    trainer = pl.Trainer(max_epochs=3, callbacks=[chkpt], default_root_dir=path, logger=logger)
    trainer.fit(f, dataloader)
