from decimal import Decimal

from prism_agents.agents.cst_agent_torch.cst_consts import PAPER_CST_PARAMS
from prism_agents.agents.cst_agent_torch.cst_remote.cst_remote_core import CSTMarketParseParams
from prism_agents.agents.cst_agent_torch.cst_remote.cst_runner import CSTRunner
from prism_agents.agents.forecast.forecasters.moving_average import (
    simple_moving_average_agent_runner,
)
from prism_agents.api.remote_request_funcs import RemoteServerInterface
from prism_agents.runner.experiment_runner import ExperimentSpec, run_experiment

test_server_url = "http://peter-test13-server-route-a5w0-photon.apps.borealis-rnd-scc.fg.rbc.com"
# test_server_url = "http://10.211.190.75:1603"


def go():
    runner_1 = CSTRunner(
        params=PAPER_CST_PARAMS,
        parse_params=CSTMarketParseParams(num_ticks=25, tick_size=Decimal("0.01"), volume_unit=1),
        seed=12345678,
        alias="peter",
    )
    runner_2 = simple_moving_average_agent_runner(
        alpha_short=1.0, alpha_long=0.5, horizon=1, volume=10, alias="moving_average"
    )
    server_interface = RemoteServerInterface(server_url=test_server_url)
    result = run_experiment(
        agents=[runner_1, runner_2],
        experiment_spec=ExperimentSpec(duration=10, time_warp_factor=1, num_observations=10),
        server_interface=server_interface,
    )
    print(result)


if __name__ == "__main__":
    go()
