from math import pi
from pathlib import Path

import numpy as np

from prism_agents.agents.forecast.forecast_agent import ForecastRunner
from prism_agents.agents.forecast.forecasters.fundamentalist.fund_forecaster import Fundamentalist
from prism_agents.agents.forecast.forecasters.fundamentalist.process.sinusoid import Sinusoid
from prism_agents.agents.forecast.forecasters.refitting.auto_arima import AutoArimaForecaster
from prism_agents.agents.forecast.trading_rule.spread_trading_rule import SpreadTradingRule
from prism_agents.api.local_server.local_server_interface import LocalServerInterface
from prism_agents.plotting.market_history_plot import basic_summary_plots
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple


def arima_experiment(out_dir: Path):
    alias_sinusoid = "sinusoid"
    alias_auto = "auto"
    duration = 1.5
    center = 100
    amplitude = 20
    sinusoid_forecast = Fundamentalist(
        start_state=Sinusoid(
            step=0, amplitude=amplitude, frequency=1 / 10, center=center, phase=-pi / 2
        )
    )
    rule = SpreadTradingRule(
        volume=10,
    )
    sinusoid_trader = ForecastRunner(
        forecaster=sinusoid_forecast,
        trading_rule=rule,
        horizon=1,
        symbol="PRSM",
        alias=alias_sinusoid,
        period=0.01,
    )
    limit = 1
    arima_period = 0.01
    auto_forecater = AutoArimaForecaster()
    auto_trader = ForecastRunner(
        forecaster=auto_forecater,
        trading_rule=SpreadTradingRule(volume=1, position_min=-limit, position_max=limit),
        horizon=2,
        symbol="PRSM",
        alias=alias_auto,
        period=arima_period,
    )
    result = run_experiment_simple(
        [sinusoid_trader, auto_trader],
        experiment_spec=SimpleExperimentSpec(
            duration=duration,
            observation_times=list(np.arange(0, duration, arima_period)),
        ),
        server_interface=LocalServerInterface(),
    )
    plots = basic_summary_plots(result)
    save_plots_to_dir(plots, folder=out_dir)


if __name__ == "__main__":
    arima_experiment(Path("/data/user/q4xwsdt/prism_test_results/v20220616/arima_plots"))
