from pathlib import Path
from typing import Optional

import numpy as np

from prism_agents.agents.forecast.forecasters.fundamentalist.fund_forecaster import (
    create_profile_fundamentalist,
)
from prism_agents.agents.forecast.forecasters.fundamentalist.process.ornstein_uhlenbeck import (
    OrnsteinUhlenbeckState,
)
from prism_agents.api.local_server.local_server_interface import LocalServerInterface
from prism_agents.plotting.market_history_plot import basic_summary_plots
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple


def volume_experiment(out_dir: Optional[Path] = None):
    duration = 14
    ou_trader = create_profile_fundamentalist(
        initial_state=OrnsteinUhlenbeckState.create_with_half_life(
            half_life=2, sigma=5, step=0, value=50, b=50
        ),
        volume=10,
        period=0.01,
    )
    result = run_experiment_simple(
        [ou_trader],
        experiment_spec=SimpleExperimentSpec(
            duration=duration,
            observation_times=list(np.arange(0, duration, 0.01)),
        ),
        server_interface=LocalServerInterface(),
    )
    plots = basic_summary_plots(result)
    save_plots_to_dir(plots, folder=out_dir)


if __name__ == "__main__":
    out_path = Path("/data/user/q4xwsdt/prism_test_results/v20220621/volume_profile_experiment")
    volume_experiment(out_dir=out_path)
