import cProfile
import pstats
from pathlib import Path
from typing import Optional

import numpy as np

from prism_agents.agents.forecast.forecasters.fundamentalist.fund_forecaster import (
    create_profile_fundamentalist,
)
from prism_agents.agents.forecast.forecasters.fundamentalist.process.brownian import (
    brownian_transition_func,
)
from prism_agents.agents.forecast.forecasters.fundamentalist.process.ornstein_uhlenbeck import (
    ou_transition_function_reset_mean,
)
from prism_agents.agents.forecast.forecasters.fundamentalist.process.regime_switching import (
    RegimeSwitchingState,
)
from prism_agents.api.local_server.local_server_interface import LocalServerInterface
from prism_agents.plotting.market_history_plot import basic_summary_plots
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple


def rs_experiment(out_dir: Optional[Path] = None):
    duration = 14
    matrix = np.array([[0.98, 0.01, 0.01], [0.01, 0.98, 0.01], [0.01, 0.01, 0.98]])
    switching_trader = create_profile_fundamentalist(
        initial_state=RegimeSwitchingState.create(
            transition_matrix=matrix,
            regime_transition_functions=[
                ou_transition_function_reset_mean(1 / 2, sigma=5),
                brownian_transition_func(mu=0.1, sigma=1),
                brownian_transition_func(mu=-0.1, sigma=1),
            ],
            starting_value=500,
        ),
        volume=10,
        period=0.01,
    )
    result = run_experiment_simple(
        [switching_trader],
        experiment_spec=SimpleExperimentSpec(
            duration=duration,
            observation_times=list(np.arange(0, duration, 0.01)),
        ),
        server_interface=LocalServerInterface(),
    )
    if out_dir is not None:
        plots = basic_summary_plots(result)
        save_plots_to_dir(plots, folder=out_dir)


prof_stats_name = "rs_prof_stats_base_v2.prof"


def run_with_profile():
    cProfile.run("rs_experiment()", prof_stats_name)


def load_prof():
    p = pstats.Stats(prof_stats_name)
    p.sort_stats("cumulative").print_stats(40)


if __name__ == "__main__":
    run_with_profile()
    load_prof()
