from pathlib import Path
from typing import Dict

import numpy as np
from plotly.graph_objs import Figure

from prism_agents.agents.cst_agent_torch.cst_consts import PAPER_CST_PARAMS
from prism_agents.agents.cst_agent_torch.cst_remote.cst_remote_core import CSTMarketParseParams
from prism_agents.agents.cst_agent_torch.cst_remote.cst_runner import CSTRunner
from prism_agents.api.local_server.local_server_interface import LocalServerInterface
from prism_agents.plotting.market_history_plot import snapshot_plot
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple


def go() -> Dict[str, Figure]:
    duration = 30
    runner_1 = CSTRunner(
        params=PAPER_CST_PARAMS,
        parse_params=CSTMarketParseParams(num_ticks=25),
        seed=12345678,
        alias="cst",
    )

    result = run_experiment_simple(
        [runner_1],
        experiment_spec=SimpleExperimentSpec(
            duration=duration,
            observation_times=list(np.arange(0, duration, 0.01)),
        ),
        server_interface=LocalServerInterface(),
    )
    print("done experiment")
    # plots = basic_summary_plots(result)
    plots = {}
    plots.update(snapshot_plot(result))
    out_dir = Path("/data/user/q4xwsdt/prism_test_results/v20220627/snapshot_cst_plots")
    save_plots_to_dir(plots, folder=out_dir)
    return plots


if __name__ == "__main__":
    go()
