from pathlib import Path
from typing import Dict

import numpy as np
from plotly.graph_objs import Figure

from prism_agents.agents.forecast.forecasters.fundamentalist.process.m4_process import (
    create_basic_m4_fundamental_agent,
)
from prism_agents.api.local_server.local_server_interface import LocalServerInterface
from prism_agents.plotting.market_history_plot import snapshot_plot
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple


def go() -> Dict[str, Figure]:
    duration = 30
    period = 0.01
    runner_1 = create_basic_m4_fundamental_agent(
        required_steps=int(duration / period) + 1, seed=42, period=period
    )

    result = run_experiment_simple(
        [runner_1],
        experiment_spec=SimpleExperimentSpec(
            duration=duration,
            observation_times=list(np.arange(0, duration, 0.01)),
        ),
        server_interface=LocalServerInterface(),
    )
    print("done experiment")
    plots = {}
    plots.update(snapshot_plot(result))
    out_dir = Path("/data/user/q4xwsdt/prism_test_results/v20220629/m4_experiment")
    save_plots_to_dir(plots, folder=out_dir)
    return plots


if __name__ == "__main__":
    go()
