from pathlib import Path
from typing import Dict

import numpy as np
from plotly.graph_objs import Figure

from prism_agents.agents.background.annealing_schedule import ExponentialAnnealingState
from prism_agents.agents.forecast.forecasters.fundamentalist.process.m4_process import (
    create_poisson_m4_fundamental_agent,
)
from prism_agents.api.local_server.local_server_interface import LocalServerInterface
from prism_agents.plotting.market_history_plot import snapshot_plot
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple


def go() -> Dict[str, Figure]:
    duration = 60 * 15
    fundamental_period = 0.5
    runner_1 = create_poisson_m4_fundamental_agent(
        required_steps=int(duration / fundamental_period) + 1,
        seed=42,
        fundamental_period=fundamental_period,
        annealer=ExponentialAnnealingState.create_with_half_life(duration / fundamental_period / 4),
        limit_arrival_rate=3,
    )

    result = run_experiment_simple(
        [runner_1],
        experiment_spec=SimpleExperimentSpec(
            duration=duration,
            observation_times=list(np.arange(0, duration, 1)),
        ),
        server_interface=LocalServerInterface(),
    )
    print("done experiment")
    plots = {}
    plots.update(snapshot_plot(result))
    out_dir = Path("/data/user/q4xwsdt/prism_test_results/v20220715/m4_experiment_poisson_anneal")
    save_plots_to_dir(plots, folder=out_dir)
    return plots


if __name__ == "__main__":
    go()
