from pathlib import Path
from typing import Dict

import numpy as np
from plotly.graph_objs import Figure

from prism_agents.agents.utils.agent_creation.ou_regime_switching import (
    create_simple_ou_regime_switching_agent,
)
from prism_agents.api.local_server.local_server_interface import LocalServerInterface
from prism_agents.plotting.market_history_plot import basic_summary_plots, snapshot_plot
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple


def go() -> Dict[str, Figure]:
    duration = 30
    period = 0.01
    trader = create_simple_ou_regime_switching_agent(
        seed=42, sigma=(0.03,) * 3, alpha=(1 / 10,) * 3, mean=(10, 10.1, 10.5), period=period
    )
    result = run_experiment_simple(
        [trader],
        experiment_spec=SimpleExperimentSpec(
            duration=duration,
            observation_times=list(np.arange(0, duration, 0.01)),
        ),
        server_interface=LocalServerInterface(),
    )
    print("done experiment")
    plots = basic_summary_plots(result)
    plots.update(snapshot_plot(result))
    out_dir = Path("/data/user/q4xwsdt/prism_test_results/v20220630/ou_regime_experiment")
    save_plots_to_dir(plots, folder=out_dir)
    return plots


if __name__ == "__main__":
    go()
