from pathlib import Path
from typing import Dict

from plotly.graph_objs import Figure

from prism_agents.agents.forecast.forecast_agent import ForecastRunner
from prism_agents.agents.forecast.forecasters.refitting.auto_arima import AutoArimaForecaster
from prism_agents.agents.forecast.trading_rule.simple_trading_rule import SimpleTradingRule
from prism_agents.agents.utils.agent_creation.ou import create_ou_poisson_agent
from prism_agents.api.remote_request_funcs import RemoteServerInterface
from prism_agents.plotting.market_history_plot import basic_summary_plots, snapshot_plot
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.experiment_runner import ExperimentSpec, run_experiment

TEST_SERVER_URl = "http://peter-test16-server-route-a5w0-photon.apps.borealis-rnd-scc.fg.rbc.com"


def go() -> Dict[str, Figure]:
    duration = 60 * 7
    fundamental_period = 0.5
    limit_arrival_rate = 10
    runner_1 = create_ou_poisson_agent(
        seed=42,
        fundamental_period=fundamental_period,
        limit_arrival_rate=limit_arrival_rate,
        half_life=5,
        sigma=0.06,
    )
    limit = 1

    auto_forecater = AutoArimaForecaster()
    arima_trader = ForecastRunner(
        forecaster=auto_forecater,
        trading_rule=SimpleTradingRule(position_max=1, position_min=-limit, volume=limit),
        horizon=2,
        symbol="PRSM",
        alias="arima",
        period=fundamental_period,
    )
    server = RemoteServerInterface(TEST_SERVER_URl)
    server.clear()
    result = run_experiment(
        [runner_1, arima_trader],
        experiment_spec=ExperimentSpec(duration=duration, num_observations=50, time_warp_factor=1),
        server_interface=server,
    )
    print("done experiment")
    plots = basic_summary_plots(result)
    plots.update(snapshot_plot(result))
    out_dir = Path("/data/user/q4xwsdt/prism_test_results/v20220704/ou_poisson_arima_remote")
    save_plots_to_dir(plots, folder=out_dir)
    return plots


if __name__ == "__main__":
    go()
