from pathlib import Path
from typing import Dict

from plotly.graph_objs import Figure

from prism_agents.agents.forecast.forecasters.fundamentalist.process.m4_process import (
    create_poisson_m4_fundamental_agent,
)
from prism_agents.api.remote_request_funcs import RemoteServerInterface
from prism_agents.plotting.market_history_plot import basic_summary_plots, snapshot_plot
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.experiment_runner import ExperimentSpec, run_experiment

TEST_SERVER_URl = "http://peter-test16-server-route-a5w0-photon.apps.borealis-rnd-scc.fg.rbc.com"


def go() -> Dict[str, Figure]:
    duration = 60 * 15
    fundamental_period = 10

    trader = create_poisson_m4_fundamental_agent(
        required_steps=int(duration / fundamental_period) + 1,
        seed=42,
        fundamental_period=fundamental_period,
    )
    server_interface = RemoteServerInterface(server_url=TEST_SERVER_URl)
    server_interface.clear()
    result = run_experiment(
        [trader],
        experiment_spec=ExperimentSpec(
            duration=duration, time_warp_factor=1.0, num_observations=10
        ),
        server_interface=server_interface,
    )
    print("done experiment")
    plots = basic_summary_plots(result)
    plots.update(snapshot_plot(result))
    out_dir = Path("/data/user/q4xwsdt/prism_test_results/v20220704/poisson_remote")
    save_plots_to_dir(plots, folder=out_dir)
    return plots


if __name__ == "__main__":
    go()
