from pathlib import Path
from typing import Dict

import numpy as np
from plotly.graph_objs import Figure

from prism_agents.agents.utils.agent_creation.ko import create_ko_poisson_agent
from prism_agents.api.local_server.local_server_interface import LocalServerInterface
from prism_agents.plotting.market_history_plot import basic_summary_plots
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple


def go() -> Dict[str, Figure]:
    duration = 60 * 7
    fundamental_period = 0.1
    limit_arrival_rate = 10
    runner_1 = create_ko_poisson_agent(
        seed=66,
        sigma=0.02,
        start_value=30,
        drift_alpha=np.log(2) / 100,
        drift_b=0.0,
        drift_sigma=0.02,
        fundamental_period=fundamental_period,
        limit_arrival_rate=limit_arrival_rate,
    )

    result = run_experiment_simple(
        [
            runner_1,
        ],
        experiment_spec=SimpleExperimentSpec(
            duration=duration,
            observation_times=list(np.arange(0, duration, 1)),
        ),
        server_interface=LocalServerInterface(),
    )
    print("done experiment")
    plots = basic_summary_plots(result)
    # plots = {}
    # plots.update(snapshot_plot(result, bounds=(Decimal(0), Decimal(20))))
    out_dir = Path("/data/user/q4xwsdt/prism_test_results/v20220706/ko_poisson_local_4")
    save_plots_to_dir(plots, folder=out_dir)
    return plots


if __name__ == "__main__":
    go()
