import cProfile
import pstats
from decimal import Decimal
from pathlib import Path
from typing import Dict

import numpy as np
from plotly.graph_objs import Figure

from prism_agents.agents.background.poisson_fundamental_trader import PoissonRegion
from prism_agents.agents.forecast.forecasters.fundamentalist.process.m4_process import (
    create_poisson_m4_fundamental_agent_multi_part,
)
from prism_agents.api.local_server.local_server_interface import LocalServerInterface
from prism_agents.plotting.market_history_plot import snapshot_plot
from prism_agents.plotting.plot_util import save_plots_to_dir
from prism_agents.runner.simple_experiment_runner import SimpleExperimentSpec, run_experiment_simple


def go(plot: bool = False) -> Dict[str, Figure]:
    duration = 60 * 15
    fundamental_period = 0.5
    half_life = duration / fundamental_period / 4
    runner_1 = create_poisson_m4_fundamental_agent_multi_part(
        required_steps=int(duration / fundamental_period) + 1,
        seed=42,
        fundamental_period=fundamental_period,
        regions=[
            PoissonRegion.create_with_half_life(arrival_rate=10, sz=10, half_life=half_life),
            PoissonRegion.create_with_half_life(arrival_rate=10, sz=30, half_life=half_life),
            PoissonRegion(arrival_rate=3, volume=3, sz=10),
        ],
    )

    result = run_experiment_simple(
        [runner_1],
        experiment_spec=SimpleExperimentSpec(
            duration=duration,
            observation_times=list(np.arange(0, duration, 1)),
        ),
        server_interface=LocalServerInterface(),
    )
    print("done experiment")
    plots = {}
    if plot:
        plots.update(snapshot_plot(result, bounds=(Decimal(8), Decimal(12))))
        out_dir = Path("/data/user/q4xwsdt/prism_test_results/v20220721/restricted_2")
        save_plots_to_dir(plots, folder=out_dir)
    return plots


prof_stats_name = "prof_stats_multi"


def run_with_profile():
    cProfile.run("go()", prof_stats_name)


def load_prof():
    p = pstats.Stats(prof_stats_name)
    p.sort_stats("cumulative").print_stats(40)


if __name__ == "__main__":
    go(True)
