# Tutorials
This directory contains scripts showcasing the capabilities of `prism-agents`.

# Tutorials List

## Tutorial A: The Server Interface
Our [first tutorial](tutorial_a_server_interface.ipynb) introduces `ServerInterface`, a convenient wrapper around calls to the Prism server.
An extra [tutorial](tutorial_a_2_barebones_api.ipynb) in the same family explains the basic functionality of the Prism API via HTTP requests
for those that prefer a more barebones approach.

## Tutorial B: Agents
Our [second tutorial](tutorial_b_agents.ipynb) introduces `StepwiseAgentRunner`, a base class that makes it easy to implement an automated trading strategy.

## Tutorial C: Forecasters and Trading Rules
Our [third tutorial](tutorial_c_trading_rules_and_forecasters.ipynb) introduces forecasters and trading rules, which can be used
to conveniently construct `StepwiseAgentRunners` that make a prediction and then trade based on this prediction.

## Tutorial D: Experiments and Analysis
Our [fourth tutorial](tutorial_d_experiments_tutorial.ipynb) describes our experiment framework, which can be used to evaluate the performance of trading agents.

Note that Prism exposes endpoints to query historical data from the server. Analyses of agents performance can 
then be performed on this data. We supply a second jupyter notebook titled [tutorial_d_2_interactive_episode_analysis.ipynb](tutorial_d_2_interactive_episode_analysis.ipynb)  
that shows how to query such data, turn it into an easy to use Pandas Dataframe and perform visualizations with plotly.

## Tutorial E: Running a Prism Agent for the Competition
This can serve as an easy to modify script that makes your agents competition ready. 

The creation and running of a trading agent based on an Arima forecast can be 
found under [`single_prism_agent.py`](tutorial_e_single_prism_agent.py).
Note that the agent will not trade if its trading rule condition is not met. 

## Tutorial F: Running many agents
You may want to test your own agents against other pre-specified agents. This tutorial is similar in spirit
to tutorial D, but within a python script. 

Functions that can help
with the task can be found in [`forecasting_agents_on_prism.py`](tutorial_f_forecasting_agents_on_prism.py) where we create multiple
different types of agents (a fundamental process to drive some dynamics and 3 
forecast traders) and run them asynchronously on a specified Prism server.

Readers interested in seeing an adaptive trader based on a neural network (LSTM) forecaster will find
such model being created and ran in this script.

# FAQs


## A Note on Fundamental Agents

We note that our tutorials sometime contain a ***Fundamental Agent***. This agent is not considered a 
normal market participant, rather it acts as a liquidity provider and a driver of price dynamics. 
Fundamental agents model the exogeneous information that typically drives market prices. 

We offer example
[fundamental agents](../agents/forecast/forecasters/fundamentalist/process)
based on stochastic processes such as Brownian Motion, 
Ornstein-Uhlenbeck, Merton Jump Diffusion, etc...

The inclusion of these processes alleviate the cold start problem of new Prism environments.

One particular feature of interest is how the volume traded by the fundamental trader varies over time.  
This is controlled by an [AnnealingState](../agents/background/annealing_schedule.py) passed to a fundamental agent constructor.  For example, in [`forecasting_agents_on_prism.py`](./tutorial_f_forecasting_agents_on_prism.py) we use an ExponentialAnnealingState.

## Alleviating the cold start problem. How should I pretrain my model?
Machine learning based forecasters may benefit from offline training before running on Prism.

We offer two types of dataset functionality available for pretraining. The first is based on
the stochastic processes available to produce fundamental agents. An example of this is given in
the [`pretrain_lstm.py`](pretrain_lstm.py) script.

The other is a dataset() class that turns a Prism market history into a torch Dataset.
